// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file SmootherBase.h */

#ifndef AMDIS_JACOBISMOOTHER_H
#define AMDIS_JACOBISMOOTHER_H

#include "SmootherBase.h"
#include "MemoryManager.h"
#include "Parameters.h"

namespace AMDiS {

  template<typename MatrixType, typename VectorType, typename DOFSetType>
  class JacobiSmoother : public SmootherBase<MatrixType, VectorType, DOFSetType>
  {
  public:
    MEMORY_MANAGED(JacobiSmoother<MatrixType COMMA VectorType COMMA DOFSetType>);

    class Creator : public SmootherCreator<MatrixType, VectorType, DOFSetType>
    {
    public:
      MEMORY_MANAGED(Creator);

      SmootherBase<MatrixType, VectorType, DOFSetType> *create() {
	return NEW JacobiSmoother<MatrixType, VectorType, DOFSetType>(this->name);
      };
    };

    JacobiSmoother(::std::string name) 
      : SmootherBase<MatrixType, VectorType, DOFSetType>(name),
	omega_(1.0)
    {
      GET_PARAMETER(0, name + "->omega", "%f", &omega_);
    };

    void smooth(MatrixType       *matrix,
		VectorType       *solution,
		VectorType       *rhs,
		int               iterations,
		const DOFSetType &dofSet);

  protected:
    double omega_;
  };

}

#endif
