// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file ResidualParallelEstimator.h */

/** \defgroup Estimator Estimator module
 * @{ <img src="estimator.png"> @}
 */

#ifndef AMDIS_RESIDUALPARALLELESTIMATOR_H
#define AMDIS_RESIDUALPARALLELESTIMATOR_H

#ifdef _OPENMP

#include "ResidualEstimator.h"
#include <vector>

namespace AMDiS {

  // ============================================================================
  // ===== class ResidualParallelEstimator ======================================
  // ============================================================================

  /**
   * \ingroup Estimator
   * 
   * \brief
   * Parallel Residual Estimator for scalar problems.
   */
  class ResidualParallelEstimator : public Estimator
  {
  public:
    MEMORY_MANAGED(ResidualParallelEstimator);
   
    /** \brief
     * Creator class used in the OEMSolverMap.
     */
    class Creator : public EstimatorCreator
    {
    public:
      MEMORY_MANAGED(Creator);

      Creator() : EstimatorCreator() {};

      virtual ~Creator() {};

      /** \brief
       * Returns a new ODirSolver object.
       */   
      Estimator* create() { 
	return NEW ResidualParallelEstimator(name, row);
      };
    };

    /** \brief
     * Constructor.
     */
    ResidualParallelEstimator(::std::string name, int r);

    /** \brief
     * Destructor.
     */
    ~ResidualParallelEstimator();

    /** \brief
     *
     */
    virtual void addSystem(DOFMatrix *matrix_,
			   DOFVector<double> *uh_,
			   DOFVector<double> *fh_,
			   DOFVector<double> *uhOld_ = NULL);

    /** \brief
     *
     */
    virtual void addUhOldToSystem(int system, DOFVector<double> *uhOld_);

    /** \brief
     *
     */
    virtual double estimate(double timestep);

  private:
    ::std::vector<ResidualEstimator*> seqEstimators_;


    /** \brief
     * Constant in fromt of the time
     */
    double C3;
  };
  
}

#endif  // _OPENMP

#endif // AMDIS_RESIDUALPARALLELESTIMATOR_H
