#include "parareal/ProblemBase.h"
#include "parareal/AdaptParaReal.h"

namespace AMDiS {
  int AdaptParaReal::adapt()
  {    
    int coarseStages = static_cast<int>(round(adaptInfo_->getEndTime() / coarseTimestep));
    int fineStages = static_cast<int>(round(coarseTimestep / fineTimestep));

    adaptInfo_->setTimestep(coarseTimestep);
    pararealProb->setStoreSolutions(false, true);
    pararealProb->setStoreInitSolution(true);
    AdaptInstationary::adapt();

    for (int k = 0; k < pararealIter; k++) {
      for (int i = 0; i < coarseStages; i++) {
	adaptInfo_->resetTimeValues(fineTimestep,
				    i * coarseTimestep,
				    (i + 1) *coarseTimestep);
	pararealProb->setStoreSolutions(true, false);
	pararealProb->setStoreInitSolution(false);
	pararealProb->setInitSolutionVec((*pararealProb->getCoarseSolutions())[i]);
	pararealProb->cleanUp();
	
	AdaptInstationary::adapt();
      }
      
      std::vector<DOFVector<double> *> coarseSolutions =
	(*pararealProb->getCoarseSolutions());
      pararealProb->clearCoarseSolutions();
      
      
      (*pararealProb->getCoarseSolutions()).push_back(coarseSolutions[0]);
      
      for (int i = 1; i <= coarseStages; i++) {
	adaptInfo_->resetTimeValues(coarseTimestep,
				    (i - 1) * coarseTimestep,
				    i * coarseTimestep);
	pararealProb->setStoreSolutions(false, true);
	pararealProb->setStoreInitSolution(false);
	pararealProb->setInitSolutionVec((*pararealProb->getCoarseSolutions())[i - 1]);
	
	AdaptInstationary::adapt();
	
	DOFVector<double>* tmp = (*pararealProb->getCoarseSolutions())[i];
	*tmp += *(*pararealProb->getFineSolutions())[(i * fineStages) - 1];
	*tmp -= *(coarseSolutions[i]);
      }
    }
      
    return 0;
  }

}
